@@ -15,9 +15,10 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  *
- * $Id: apdu.c,v 1.19 2005/01/25 14:18:56 wk Exp $
+ * $Id: apdu.c 3753 2005-05-31 08:39:18Z wk $
  */
 
 #include <config.h>
@@ -150,12 +151,12 @@ static struct reader_table_s reader_table[MAX_READER];
 
 
 /* ct API function pointer. */
-static char (* DLSTDCALL CT_init) (unsigned short ctn, unsigned short Pn);
-static char (* DLSTDCALL CT_data) (unsigned short ctn, unsigned char *dad,
+static char (DLSTDCALL * CT_init) (unsigned short ctn, unsigned short Pn);
+static char (DLSTDCALL * CT_data) (unsigned short ctn, unsigned char *dad,
                                    unsigned char *sad, unsigned short lc,
                                    unsigned char *cmd, unsigned short *lr,
                                    unsigned char *rsp);
-static char (* DLSTDCALL CT_close) (unsigned short ctn);
+static char (DLSTDCALL * CT_close) (unsigned short ctn);
 
 /* PC/SC constants and function pointer. */
 #define PCSC_SCOPE_USER      0 
@@ -217,46 +218,46 @@ struct pcsc_readerstate_s
 
 typedef struct pcsc_readerstate_s *pcsc_readerstate_t;
 
-long (* DLSTDCALL pcsc_establish_context) (unsigned long scope,
+long (DLSTDCALL * pcsc_establish_context) (unsigned long scope,
                                            const void *reserved1,
                                            const void *reserved2,
                                            unsigned long *r_context);
-long (* DLSTDCALL pcsc_release_context) (unsigned long context);
-long (* DLSTDCALL pcsc_list_readers) (unsigned long context,
+long (DLSTDCALL * pcsc_release_context) (unsigned long context);
+long (DLSTDCALL * pcsc_list_readers) (unsigned long context,
                                       const char *groups,
                                       char *readers, unsigned long*readerslen);
-long (* DLSTDCALL pcsc_get_status_change) (unsigned long context,
+long (DLSTDCALL * pcsc_get_status_change) (unsigned long context,
                                            unsigned long timeout,
                                            pcsc_readerstate_t readerstates,
                                            unsigned long nreaderstates);
-long (* DLSTDCALL pcsc_connect) (unsigned long context,
+long (DLSTDCALL * pcsc_connect) (unsigned long context,
                                  const char *reader,
                                  unsigned long share_mode,
                                  unsigned long preferred_protocols,
                                  unsigned long *r_card,
                                  unsigned long *r_active_protocol);
-long (* DLSTDCALL pcsc_reconnect) (unsigned long card,
+long (DLSTDCALL * pcsc_reconnect) (unsigned long card,
                                    unsigned long share_mode,
                                    unsigned long preferred_protocols,
                                    unsigned long initialization,
                                    unsigned long *r_active_protocol);
-long (* DLSTDCALL pcsc_disconnect) (unsigned long card,
+long (DLSTDCALL * pcsc_disconnect) (unsigned long card,
                                     unsigned long disposition);
-long (* DLSTDCALL pcsc_status) (unsigned long card,
+long (DLSTDCALL * pcsc_status) (unsigned long card,
                                 char *reader, unsigned long *readerlen,
                                 unsigned long *r_state,
                                 unsigned long *r_protocol,
                                 unsigned char *atr, unsigned long *atrlen);
-long (* DLSTDCALL pcsc_begin_transaction) (unsigned long card);
-long (* DLSTDCALL pcsc_end_transaction) (unsigned long card);
-long (* DLSTDCALL pcsc_transmit) (unsigned long card,
+long (DLSTDCALL * pcsc_begin_transaction) (unsigned long card);
+long (DLSTDCALL * pcsc_end_transaction) (unsigned long card);
+long (DLSTDCALL * pcsc_transmit) (unsigned long card,
                                   const pcsc_io_request_t send_pci,
                                   const unsigned char *send_buffer,
                                   unsigned long send_len,
                                   pcsc_io_request_t recv_pci,
                                   unsigned char *recv_buffer,
                                   unsigned long *recv_len);
-long (* DLSTDCALL pcsc_set_timeout) (unsigned long context,
+long (DLSTDCALL * pcsc_set_timeout) (unsigned long context,
                                      unsigned long timeout);
 
 
