@@ -1,5 +1,5 @@
 /* exec.c - generic call-a-program code
- * Copyright (C) 2001, 2002 Free Software Foundation, Inc.
+ * Copyright (C) 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -15,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -51,7 +52,7 @@ int exec_write(struct exec_info **info,const char *program,
 
 int exec_read(struct exec_info *info) { return G10ERR_GENERAL; }
 int exec_finish(struct exec_info *info) { return G10ERR_GENERAL; }
-int set_exec_path(const char *path,int method) { return G10ERR_GENERAL; }
+int set_exec_path(const char *path) { return G10ERR_GENERAL; }
 
 #else /* ! NO_EXEC */
 
@@ -59,7 +60,7 @@ int set_exec_path(const char *path,int method) { return G10ERR_GENERAL; }
 char *mkdtemp(char *template);
 #endif
 
-#if defined (__MINGW32__)
+#if defined (_WIN32)
 /* This is a nicer system() for windows that waits for programs to
    return before returning control to the caller.  I hate helpful
    computers. */
@@ -91,29 +92,17 @@ static int win_system(const char *command)
 }
 #endif
 
-/* method==0 to replace current $PATH, and 1 to append to current
-   $PATH.  */
-int set_exec_path(const char *path,int method)
+/* Replaces current $PATH */
+int set_exec_path(const char *path)
 {
-  char *p,*curpath=NULL;
-  size_t curlen=0;
+  char *p;
 
-  if(method==1 && (curpath=getenv("PATH")))
-    curlen=strlen(curpath)+1;
-
-  p=m_alloc(5+curlen+strlen(path)+1);
+  p=m_alloc(5+strlen(path)+1);
   strcpy(p,"PATH=");
-
-  if(curpath)
-    {
-      strcat(p,curpath);
-      strcat(p,":");
-    }
-
   strcat(p,path);
 
   if(DBG_EXTPROG)
-    log_debug("set_exec_path method %d: %s\n",method,p);
+    log_debug("set_exec_path: %s\n",p);
 
   /* Notice that path is never freed.  That is intentional due to the
      way putenv() works.  This leaks a few bytes if we call
@@ -139,7 +128,7 @@ static int make_tempdir(struct exec_info *info)
 
   if(tmp==NULL)
     {
-#if defined (__MINGW32__)
+#if defined (_WIN32)
       tmp=m_alloc(256);
       if(GetTempPath(256,tmp)==0)
 	strcpy(tmp,"c:\\windows\\temp");
@@ -176,7 +165,7 @@ static int make_tempdir(struct exec_info *info)
 
   sprintf(info->tempdir,"%s" DIRSEP_S "gpg-XXXXXX",tmp);
 
-#if defined (__MINGW32__)
+#if defined (_WIN32)
   m_free(tmp);
 #endif
 
@@ -352,8 +341,8 @@ int exec_write(struct exec_info **info,const char *program,
 #ifdef EXEC_TEMPFILE_ONLY
   if(!(*info)->use_temp_files)
     {
-      log_error(_("this platform requires temp files when calling external "
-		  "programs\n"));
+      log_error(_("this platform requires temporary files when calling"
+		  " external programs\n"));
       goto fail;
     }
 
@@ -429,10 +418,12 @@ int exec_write(struct exec_info **info,const char *program,
 
 	  /* If we get this far the exec failed.  Clean up and return. */
 
-	  log_error(_("unable to execute %s \"%s\": %s\n"),
-		    args_in==NULL?"program":"shell",
-		    args_in==NULL?program:shell,
-		    strerror(errno));
+	  if(args_in==NULL)
+	    log_error(_("unable to execute program `%s': %s\n"),
+		      program,strerror(errno));
+	  else
+	    log_error(_("unable to execute shell `%s': %s\n"),
+		      shell,strerror(errno));
 
 	  /* This mimics the POSIX sh behavior - 127 means "not found"
              from the shell. */
@@ -475,7 +466,13 @@ int exec_write(struct exec_info **info,const char *program,
     log_debug("using temp file `%s'\n",(*info)->tempfile_in);
 
   /* It's not fork/exec/pipe, so create a temp file */
-  (*info)->tochild=fopen((*info)->tempfile_in,binary?"wb":"w");
+  if( is_secured_filename ((*info)->tempfile_in) )
+    {
+      (*info)->tochild = NULL;
+      errno = EPERM;
+    }
+  else
+    (*info)->tochild=fopen((*info)->tempfile_in,binary?"wb":"w");
   if((*info)->tochild==NULL)
     {
       log_error(_("can't create `%s': %s\n"),
@@ -502,7 +499,7 @@ int exec_read(struct exec_info *info)
       if(DBG_EXTPROG)
 	log_debug("system() command is %s\n",info->command);
 
-#if defined (__MINGW32__)
+#if defined (_WIN32)
       info->progreturn=win_system(info->command);
 #else
       info->progreturn=system(info->command);
@@ -543,6 +540,13 @@ int exec_read(struct exec_info *info)
       if(!info->writeonly)
 	{
 	  info->fromchild=iobuf_open(info->tempfile_out);
+          if (info->fromchild
+              && is_secured_file (iobuf_get_fd (info->fromchild)))
+            {
+              iobuf_close (info->fromchild);
+              info->fromchild = NULL;
+              errno = EPERM;
+            }
 	  if(info->fromchild==NULL)
 	    {
 	      log_error(_("unable to read external program response: %s\n"),
