@@ -16,7 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -54,6 +55,8 @@ struct stats_s {
     ulong secret_dups;
     ulong skipped_new_keys;
     ulong not_imported;
+    ulong n_sigs_cleaned;
+    ulong n_uids_cleaned;
 };
 
 
@@ -93,10 +96,14 @@ parse_import_options(char *str,unsigned int *options,int noisy)
       {"fast-import",IMPORT_FAST,NULL},
       {"convert-sk-to-pk",IMPORT_SK2PK,NULL},
       {"merge-only",IMPORT_MERGE_ONLY,NULL},
-      {"import-unusable-sigs",IMPORT_UNUSABLE_SIGS,NULL},
+      {"import-clean",IMPORT_CLEAN_SIGS|IMPORT_CLEAN_UIDS,NULL},
+      {"import-clean-sigs",IMPORT_CLEAN_SIGS,NULL},
+      {"import-clean-uids",IMPORT_CLEAN_UIDS,NULL},
       /* Aliases for backward compatibility */
       {"allow-local-sigs",IMPORT_LOCAL_SIGS,NULL},
       {"repair-hkp-subkey-bug",IMPORT_REPAIR_PKS_SUBKEY_BUG,NULL},
+      /* dummy */
+      {"import-unusable-sigs",0,NULL},
       {NULL,0,NULL}
     };
 
@@ -300,6 +307,10 @@ import_print_stats (void *hd)
 	    log_info(_(" secret keys unchanged: %lu\n"), stats->secret_dups );
 	if( stats->not_imported )
 	    log_info(_("          not imported: %lu\n"), stats->not_imported );
+	if( stats->n_sigs_cleaned)
+	    log_info(_("    signatures cleaned: %lu\n"),stats->n_sigs_cleaned);
+	if( stats->n_uids_cleaned)
+	    log_info(_("      user IDs cleaned: %lu\n"),stats->n_uids_cleaned);
     }
 
     if( is_status_enabled() ) {
@@ -647,6 +658,20 @@ check_prefs(KBNODE keyblock)
     }
 }
 
+static int
+clean_sigs_from_all_uids(KBNODE keyblock)
+{
+  KBNODE uidnode;
+  int deleted=0;
+
+  for(uidnode=keyblock->next;uidnode;uidnode=uidnode->next)
+    if(uidnode->pkt->pkttype==PKT_USER_ID)
+      deleted+=clean_sigs_from_uid(keyblock,uidnode,opt.verbose);
+
+  return deleted;
+}
+
+
 /****************
  * Try to import one keyblock.	Return an error only in serious cases, but
  * never for an invalid keyblock.  It uses log_error to increase the
@@ -706,6 +731,16 @@ import_one( const char *fname, KBNODE keyblock,
             return 0;
     }
 
+    /* Clean the key that we're about to import, to cut down on things
+       that we have to clean later.  This has no practical impact on
+       the end result, but does result in less logging which might
+       confuse the user. */
+    if(options&IMPORT_CLEAN_SIGS)
+      clean_sigs_from_all_uids(keyblock);
+
+    if(options&IMPORT_CLEAN_UIDS)
+      clean_uids_from_key(keyblock,opt.verbose);
+
     clear_kbnode_flags( keyblock );
 
     if((options&IMPORT_REPAIR_PKS_SUBKEY_BUG) && fix_pks_corruption(keyblock)
@@ -806,7 +841,7 @@ import_one( const char *fname, KBNODE keyblock,
     }
     else { /* merge */
         KEYDB_HANDLE hd;
-	int n_uids, n_sigs, n_subk;
+	int n_uids, n_sigs, n_subk, n_sigs_cleaned, n_uids_cleaned;
 
 	/* Compare the original against the new key; just to be sure nothing
 	 * weird is going on */
@@ -847,14 +882,22 @@ import_one( const char *fname, KBNODE keyblock,
 	/* and try to merge the block */
 	clear_kbnode_flags( keyblock_orig );
 	clear_kbnode_flags( keyblock );
-	n_uids = n_sigs = n_subk = 0;
+	n_uids = n_sigs = n_subk = n_sigs_cleaned = n_uids_cleaned = 0;
 	rc = merge_blocks( fname, keyblock_orig, keyblock,
-				keyid, &n_uids, &n_sigs, &n_subk );
-	if( rc ) {
+			   keyid, &n_uids, &n_sigs, &n_subk );
+	if( rc )
+	  {
             keydb_release (hd);
 	    goto leave;
-        }
-	if( n_uids || n_sigs || n_subk ) {
+	  }
+
+	if(options&IMPORT_CLEAN_SIGS)
+	  n_sigs_cleaned=clean_sigs_from_all_uids(keyblock_orig);
+
+        if(options&IMPORT_CLEAN_UIDS)
+	  n_uids_cleaned=clean_uids_from_key(keyblock_orig,opt.verbose);
+
+	if( n_uids || n_sigs || n_subk || n_sigs_cleaned || n_uids_cleaned) {
 	    mod_key = 1;
 	    /* keyblock_orig has been updated; write */
 	    rc = keydb_update_keyblock (hd, keyblock_orig);
@@ -886,12 +929,26 @@ import_one( const char *fname, KBNODE keyblock,
 		else if( n_subk )
 		  log_info( _("key %s: \"%s\" %d new subkeys\n"),
 			    keystr(keyid), p, n_subk );
+		if(n_sigs_cleaned==1)
+		  log_info(_("key %s: \"%s\" %d signature cleaned\n"),
+			   keystr(keyid),p,n_sigs_cleaned);
+		else if(n_sigs_cleaned)
+		  log_info(_("key %s: \"%s\" %d signatures cleaned\n"),
+			   keystr(keyid),p,n_sigs_cleaned);
+		if(n_uids_cleaned==1)
+		  log_info(_("key %s: \"%s\" %d user ID cleaned\n"),
+			   keystr(keyid),p,n_uids_cleaned);
+		else if(n_uids_cleaned)
+		  log_info(_("key %s: \"%s\" %d user IDs cleaned\n"),
+			   keystr(keyid),p,n_uids_cleaned);
 		m_free(p);
 	      }
 
 	    stats->n_uids +=n_uids;
 	    stats->n_sigs +=n_sigs;
 	    stats->n_subk +=n_subk;
+	    stats->n_sigs_cleaned +=n_sigs_cleaned;
+	    stats->n_uids_cleaned +=n_uids_cleaned;
 
             if (is_status_enabled ()) 
                  print_import_ok (pk, NULL,
@@ -1289,12 +1346,13 @@ chk_self_sigs( const char *fname, KBNODE keyblock,
 	sig = n->pkt->pkt.signature;
 	if( keyid[0] == sig->keyid[0] && keyid[1] == sig->keyid[1] ) {
 
-	  /* This just caches the sigs for later use.  That way we
-	     import a fully-cached key which speeds things up. */
-	  if(!opt.no_sig_cache)
-	    check_key_signature(keyblock,n,NULL);
+	    /* This just caches the sigs for later use.  That way we
+	       import a fully-cached key which speeds things up. */
+	    if(!opt.no_sig_cache)
+	      check_key_signature(keyblock,n,NULL);
 
-	    if( (sig->sig_class&~3) == 0x10 ) {
+	    if( IS_UID_SIG(sig) || IS_UID_REV(sig) )
+	      {
 		KBNODE unode = find_prev_kbnode( keyblock, n, PKT_USER_ID );
 		if( !unode )
 		  {
@@ -1324,7 +1382,7 @@ chk_self_sigs( const char *fname, KBNODE keyblock,
 		  else
 		    unode->flag |= 1; /* mark that signature checked */
 		}
-	    }
+	      }
 	    else if( sig->sig_class == 0x18 ) {
 	      /* Note that this works based solely on the timestamps
 		 like the rest of gpg.  If the standard gets
@@ -1555,30 +1613,6 @@ delete_inv_parts( const char *fname, KBNODE keyblock,
 		       node->pkt->pkt.signature->sig_class);
 	    delete_kbnode(node);
 	  }
-	else if(node->pkt->pkttype==PKT_SIGNATURE
-		&& IS_UID_SIG(node->pkt->pkt.signature)
-		&& node->pkt->pkt.signature->flags.expired
-		&& (node->pkt->pkt.signature->keyid[0]!=keyid[0]
-		    || node->pkt->pkt.signature->keyid[1]!=keyid[1])
-		&& !(options&IMPORT_UNUSABLE_SIGS))
-	  {
-	    /* Note that we haven't necessarily checked this sig for
-	       validity.  We strip it anyway since if it was valid, it
-	       would be expired.  If it wasn't valid, it either would
-	       be expired if it was valid, or was altered to the point
-	       that it looked expired (and so doesn't matter terribly
-	       much if it is expired or not). */
-	    if(opt.verbose)
-	      {
-		char *kid=m_strdup(keystr(keyid));
-		log_info(_("key %s: expired signature from key %s -"
-			   " skipped\n"),kid,
-			 keystr(node->pkt->pkt.signature->keyid));
-		m_free(kid);
-	      }
-
-	    delete_kbnode(node);
-	  }
 	else if( (node->flag & 4) ) /* marked for deletion */
 	  delete_kbnode( node );
     }
