@@ -1,5 +1,5 @@
 /* pkclist.c
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003,
  *               2004 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
@@ -16,7 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -39,10 +40,8 @@
 #include "photoid.h"
 #include "i18n.h"
 
-
 #define CONTROL_D ('D' - 'A' + 1)
 
-
 /****************
  * Show the revocation reason as it is stored with the given signature
  */
@@ -158,74 +157,6 @@ show_revocation_reason( PKT_public_key *pk, int mode )
 }
 
 
-static void
-show_paths (const PKT_public_key *pk, int only_first )
-{
-    log_debug("not yet implemented\n");
-#if 0    
-    void *context = NULL;
-    unsigned otrust, validity;
-    int last_level, level;
-
-    last_level = 0;
-    while( (level=enum_cert_paths( &context, &lid, &otrust, &validity)) != -1){
-	char *p;
-	int c, rc;
-	size_t n;
-	u32 keyid[2];
-	PKT_public_key *pk ;
-
-	if( level < last_level && only_first )
-	    break;
-	last_level = level;
-
-	rc = keyid_from_lid( lid, keyid );
-
-	if( rc ) {
-	    log_error("ooops: can't get keyid for lid %lu\n", lid);
-	    return;
-	}
-
-	pk = m_alloc_clear( sizeof *pk );
-	rc = get_pubkey( pk, keyid );
-	if( rc ) {
-	    log_error("key %08lX: public key not found: %s\n",
-				    (ulong)keyid[1], g10_errstr(rc) );
-	    return;
-	}
-
-	tty_printf("%*s%4u%c/%08lX.%lu %s \"",
-		  level*2, "",
-		  nbits_from_pk( pk ), pubkey_letter( pk->pubkey_algo ),
-		  (ulong)keyid[1], lid, datestr_from_pk( pk ) );
-
-	c = trust_letter(otrust);
-	if( c )
-	    putchar( c );
-	else
-	    printf( "%02x", otrust );
-	putchar('/');
-	c = trust_letter(validity);
-	if( c )
-	    putchar( c );
-	else
-	    printf( "%02x", validity );
-	putchar(' ');
-
-	p = get_user_id( keyid, &n );
-	tty_print_utf8_string( p, n ),
-	m_free(p);
-	tty_printf("\"\n");
-	free_public_key( pk );
-    }
-    enum_cert_paths( &context, NULL, NULL, NULL ); /* release context */
-#endif
-    tty_printf("\n");
-}
-
-
-
-
 /****************
  * mode: 0 = standard
  *       1 = Without key info and additional menu option 'm'
@@ -241,16 +172,37 @@ do_edit_ownertrust (PKT_public_key *pk, int mode,
                     unsigned *new_trust, int defer_help )
 {
   char *p;
-  size_t n;
   u32 keyid[2];
   int changed=0;
   int quit=0;
   int show=0;
+  int min_num;
   int did_help=defer_help;
+  unsigned int minimum=get_min_ownertrust(pk);
+
+  switch(minimum)
+    {
+    default:              min_num=0; break;
+    case TRUST_UNDEFINED: min_num=1; break;
+    case TRUST_NEVER:     min_num=2; break;
+    case TRUST_MARGINAL:  min_num=3; break;
+    case TRUST_FULLY:     min_num=4; break;
+    }
 
   keyid_from_pk (pk, keyid);
   for(;;) {
-    /* a string with valid answers */
+    /* A string with valid answers.
+
+       Note to translators: These are the allowed answers in lower and
+       uppercase.  Below you will find the matching strings which
+       should be translated accordingly and the letter changed to
+       match the one in the answer string.
+    
+         i = please show me more information
+         m = back to the main menu
+         s = skip this key
+	 q = quit
+    */
     const char *ans = _("iImMqQsS");
 
     if( !did_help ) 
@@ -259,67 +211,87 @@ do_edit_ownertrust (PKT_public_key *pk, int mode,
           {
             KBNODE keyblock, un;
 
-            tty_printf(_("No trust value assigned to:\n"
-                         "%4u%c/%08lX %s \""),
-                       nbits_from_pk( pk ), pubkey_letter( pk->pubkey_algo ),
-                       (ulong)keyid[1], datestr_from_pk( pk ) );
-            p = get_user_id( keyid, &n );
-            tty_print_utf8_string( p, n ),
-              m_free(p);
-            tty_printf("\"\n");
+            tty_printf(_("No trust value assigned to:\n"));
+	    tty_printf("%4u%c/%s %s\n",nbits_from_pk( pk ),
+		       pubkey_letter( pk->pubkey_algo ),
+                       keystr(keyid), datestr_from_pk( pk ) );
+	    p=get_user_id_native(keyid);
+	    tty_printf(_("      \"%s\"\n"),p);
+	    m_free(p);
 
             keyblock = get_pubkeyblock (keyid);
             if (!keyblock)
                 BUG ();
-            for (un=keyblock; un; un = un->next) {
+            for (un=keyblock; un; un = un->next)
+	      {
                 if (un->pkt->pkttype != PKT_USER_ID )
-                    continue;
+		  continue;
                 if (un->pkt->pkt.user_id->is_revoked )
-                    continue;
+		  continue;
                 if (un->pkt->pkt.user_id->is_expired )
-                    continue;
+		  continue;
 		/* Only skip textual primaries */
-                if (un->pkt->pkt.user_id->is_primary &&
-		    !un->pkt->pkt.user_id->attrib_data )
-		    continue;
+                if (un->pkt->pkt.user_id->is_primary
+		    && !un->pkt->pkt.user_id->attrib_data )
+		  continue;
                 
-		if(opt.show_photos && un->pkt->pkt.user_id->attrib_data)
-                    show_photos(un->pkt->pkt.user_id->attribs,
-                                un->pkt->pkt.user_id->numattribs,pk,NULL);
-                
-		tty_printf ("      %s", _("                aka \""));
-                tty_print_utf8_string (un->pkt->pkt.user_id->name,
-                                       un->pkt->pkt.user_id->len );
-                tty_printf("\"\n");
-            }
+		if((opt.verify_options&VERIFY_SHOW_PHOTOS)
+		   && un->pkt->pkt.user_id->attrib_data)
+		  show_photos(un->pkt->pkt.user_id->attribs,
+			      un->pkt->pkt.user_id->numattribs,pk,NULL);
+
+		p=utf8_to_native(un->pkt->pkt.user_id->name,
+				 un->pkt->pkt.user_id->len,0);
+
+		tty_printf(_("  aka \"%s\"\n"),p);
+	      }
         
             print_fingerprint (pk, NULL, 2);
             tty_printf("\n");
-	    release_kbnode(keyblock);
+	    release_kbnode (keyblock);
           }
 
-        tty_printf (_(
-                     "Please decide how far you trust this user to correctly\n"
-                     "verify other users' keys (by looking at passports,\n"
-                     "checking fingerprints from different sources...)?\n\n"));
-        tty_printf (_(" %d = Don't know\n"), 1);
-        tty_printf (_(" %d = I do NOT trust\n"), 2);
-        tty_printf (_(" %d = I trust marginally\n"), 3);
-        tty_printf (_(" %d = I trust fully\n"), 4);
+	if(opt.trust_model==TM_DIRECT)
+	  {
+	    tty_printf(_("How much do you trust that this key actually "
+			 "belongs to the named user?\n"));
+	    tty_printf("\n");
+	  }
+	else
+	  {
+	    /* This string also used in keyedit.c:trustsig_prompt */
+	    tty_printf(_("Please decide how far you trust this user to"
+			 " correctly verify other users' keys\n"
+			 "(by looking at passports, checking fingerprints from"
+			 " different sources, etc.)\n"));
+	    tty_printf("\n");
+	  }
+
+	if(min_num<=1)
+	  tty_printf (_("  %d = I don't know or won't say\n"), 1);
+	if(min_num<=2)
+	  tty_printf (_("  %d = I do NOT trust\n"), 2);
+	if(min_num<=3)
+	  tty_printf (_("  %d = I trust marginally\n"), 3);
+	if(min_num<=4)
+	  tty_printf (_("  %d = I trust fully\n"), 4);
         if (mode)
-          tty_printf (_(" %d = I trust ultimately\n"), 5);
+          tty_printf (_("  %d = I trust ultimately\n"), 5);
 #if 0
 	/* not yet implemented */
-        tty_printf (_(" i = please show me more information\n") );
+        tty_printf ("  i = please show me more information\n");
 #endif
         if( mode )
-          tty_printf(_(" m = back to the main menu\n"));
+          tty_printf(_("  m = back to the main menu\n"));
         else
 	  {
-	    tty_printf(_(" s = skip this key\n"));
-	    tty_printf(_(" q = quit\n"));
+	    tty_printf(_("  s = skip this key\n"));
+	    tty_printf(_("  q = quit\n"));
 	  }
         tty_printf("\n");
+	if(minimum)
+	  tty_printf(_("The minimum trust level for this key is: %s\n\n"),
+		     trust_value_to_string(minimum));
         did_help = 1;
       }
     if( strlen(ans) != 8 )
@@ -331,7 +303,7 @@ do_edit_ownertrust (PKT_public_key *pk, int mode,
       did_help = 0;
     else if( *p && p[1] )
       ;
-    else if( !p[1] && (*p >= '1' && *p <= (mode?'5':'4')) ) 
+    else if( !p[1] && ((*p >= '0'+min_num) && *p <= (mode?'5':'4')) ) 
       {
         unsigned int trust;
         switch( *p )
@@ -346,7 +318,7 @@ do_edit_ownertrust (PKT_public_key *pk, int mode,
         if (trust == TRUST_ULTIMATE
             && !cpr_get_answer_is_yes ("edit_ownertrust.set_ultimate.okay",
                                        _("Do you really want to set this key"
-                                         " to ultimate trust? ")))
+                                         " to ultimate trust? (y/N) ")))
           ; /* no */
         else
           {
@@ -401,7 +373,6 @@ edit_ownertrust (PKT_public_key *pk, int mode )
         case -1: /* quit */
           return -1;
         case -2: /* show info */
-          show_paths(pk, 1);
           no_help = 1;
           break;
         case 1: /* trust value set */
@@ -421,94 +392,54 @@ edit_ownertrust (PKT_public_key *pk, int mode )
  * Returns: true if we trust.
  */
 static int
-do_we_trust( PKT_public_key *pk, unsigned int *trustlevel )
+do_we_trust( PKT_public_key *pk, unsigned int trustlevel )
 {
-    unsigned int trustmask = 0;
+  /* We should not be able to get here with a revoked or expired
+     key */
+  if(trustlevel & TRUST_FLAG_REVOKED
+     || trustlevel & TRUST_FLAG_SUB_REVOKED
+     || (trustlevel & TRUST_MASK) == TRUST_EXPIRED)
+    BUG();
 
-    /* FIXME: get_pubkey_byname already checks the validity and won't
-     * return keys which are either expired or revoked - so these
-     * question here won't get triggered.  We have to find a solution
-     * for this.  It might make sense to have a function in getkey.c
-     * which does only the basic checks and returns even revoked and
-     * expired keys.  This fnction could then also returhn a list of
-     * keys if the speicified name is ambiguous
-     */
-    if( (*trustlevel & TRUST_FLAG_REVOKED) ) {
-	log_info(_("key %08lX: key has been revoked!\n"),
-					(ulong)keyid_from_pk( pk, NULL) );
-	show_revocation_reason( pk, 0 );
-	if( opt.batch )
-          return 0; /* no */
-
-	if( !cpr_get_answer_is_yes("revoked_key.override",
-				    _("Use this key anyway? ")) )
-          return 0; /* no */
-	trustmask |= TRUST_FLAG_REVOKED;
-    }
-    if( (*trustlevel & TRUST_FLAG_SUB_REVOKED) ) {
-	log_info(_("key %08lX: subkey has been revoked!\n"),
-					(ulong)keyid_from_pk( pk, NULL) );
-	show_revocation_reason( pk, 0 );
-	if( opt.batch )
-	    return 0;
-
-	if( !cpr_get_answer_is_yes("revoked_key.override",
-				    _("Use this key anyway? ")) )
-	    return 0;
-	trustmask |= TRUST_FLAG_SUB_REVOKED;
-    }
-    *trustlevel &= ~trustmask;
-
-    if( opt.always_trust) {
-	if( opt.verbose )
-	    log_info("No trust check due to --always-trust option\n");
-	return 1;
+  if( opt.trust_model==TM_ALWAYS )
+    {
+      if( opt.verbose )
+	log_info("No trust check due to `--trust-model always' option\n");
+      return 1;
     }
 
-    switch( (*trustlevel & TRUST_MASK) ) {
-      case TRUST_EXPIRED:
-	log_info(_("%08lX: key has expired\n"),
-				    (ulong)keyid_from_pk( pk, NULL) );
-	return 0; /* no */
+  switch(trustlevel & TRUST_MASK)
+    {
+    default:
+      log_error ("invalid trustlevel %u returned from validation layer\n",
+		 trustlevel);
+      /* fall thru */
+    case TRUST_UNKNOWN: 
+    case TRUST_UNDEFINED:
+      log_info(_("%s: There is no assurance this key belongs"
+		 " to the named user\n"),keystr_from_pk(pk));
+      return 0; /* no */
 
-      default:
-         log_error ("invalid trustlevel %u returned from validation layer\n",
-                    *trustlevel);
-         /* fall thru */
-      case TRUST_UNKNOWN: 
-      case TRUST_UNDEFINED:
-        log_info(_("%08lX: There is no indication that this key "
-                   "really belongs to the owner\n"),
-                 (ulong)keyid_from_pk( pk, NULL) );
-	return 0; /* no */
+    case TRUST_MARGINAL:
+      log_info(_("%s: There is limited assurance this key belongs"
+		 " to the named user\n"),keystr_from_pk(pk));
+      return 1; /* yes */
 
-      case TRUST_NEVER:
-	log_info(_("%08lX: We do NOT trust this key\n"),
-					(ulong)keyid_from_pk( pk, NULL) );
-	return 0; /* no */
+    case TRUST_FULLY:
+      if( opt.verbose )
+	log_info(_("This key probably belongs to the named user\n"));
+      return 1; /* yes */
 
-      case TRUST_MARGINAL:
-	log_info(
-       _("%08lX: It is not sure that this key really belongs to the owner\n"
-	 "but it is accepted anyway\n"), (ulong)keyid_from_pk( pk, NULL) );
-	return 1; /* yes */
-
-      case TRUST_FULLY:
-	if( opt.verbose )
-	    log_info(_("This key probably belongs to the owner\n"));
-	return 1; /* yes */
-
-      case TRUST_ULTIMATE:
-	if( opt.verbose )
-	    log_info(_("This key belongs to us\n"));
-	return 1; /* yes */
+    case TRUST_ULTIMATE:
+      if( opt.verbose )
+	log_info(_("This key belongs to us\n"));
+      return 1; /* yes */
     }
 
-    return 1; /* yes */
+  return 1; /* yes */
 }
 
 
-
 /****************
  * wrapper around do_we_trust, so we can ask whether to use the
  * key anyway.
@@ -516,58 +447,34 @@ do_we_trust( PKT_public_key *pk, unsigned int *trustlevel )
 static int
 do_we_trust_pre( PKT_public_key *pk, unsigned int trustlevel )
 {
-    int rc;
+  int rc;
 
-    rc = do_we_trust( pk, &trustlevel );
+  rc = do_we_trust( pk, trustlevel );
 
-    if( (trustlevel & TRUST_FLAG_REVOKED) && !rc )
-	return 0;
-    if( (trustlevel & TRUST_FLAG_SUB_REVOKED) && !rc )
-	return 0;
+  if( !opt.batch && !rc )
+    {
+      print_pubkey_info(NULL,pk);
+      print_fingerprint (pk, NULL, 2);
+      tty_printf("\n");
 
-    if( !opt.batch && !rc ) {
-	u32 keyid[2];
+      tty_printf(
+	       _("It is NOT certain that the key belongs to the person named\n"
+		 "in the user ID.  If you *really* know what you are doing,\n"
+		 "you may answer the next question with yes.\n"));
 
-	keyid_from_pk( pk, keyid);
-	tty_printf( "%4u%c/%08lX %s \"",
-		  nbits_from_pk( pk ), pubkey_letter( pk->pubkey_algo ),
-		  (ulong)keyid[1], datestr_from_pk( pk ) );
-	/* If the pk was chosen by a particular user ID, this is the
-	   one to ask about. */
-	if(pk->user_id)
-	  tty_print_utf8_string(pk->user_id->name,pk->user_id->len);
-	else
-	  {
-	    size_t n;
-	    char *p = get_user_id( keyid, &n );
-	    tty_print_utf8_string( p, n );
-	    m_free(p);
-	  }
-	tty_printf("\"\n");
-        print_fingerprint (pk, NULL, 2);
-	tty_printf("\n");
+      tty_printf("\n");
 
-	tty_printf(_(
-"It is NOT certain that the key belongs to the person named\n"
-"in the user ID.  If you *really* know what you are doing,\n"
-"you may answer the next question with yes\n\n"));
-
-	if( cpr_get_answer_is_yes("untrusted_key.override",
-				  _("Use this key anyway? "))  )
-	    rc = 1;
-
-	/* Hmmm: Should we set a flag to tell the user about
-	 *	 his decision the next time he encrypts for this recipient?
-	 */
-    }
-    else if( opt.always_trust && !rc ) {
-	if( !opt.quiet )
-	    log_info(_("WARNING: Using untrusted key!\n"));
+      if( cpr_get_answer_is_yes("untrusted_key.override",
+				_("Use this key anyway? (y/N) "))  )
 	rc = 1;
-    }
-    return rc;
-}
 
+      /* Hmmm: Should we set a flag to tell the user about
+       *	 his decision the next time he encrypts for this recipient?
+       */
+    }
+
+  return rc;
+}
 
 
 /****************
@@ -589,7 +496,7 @@ check_signatures_trust( PKT_signature *sig )
       goto leave;
     }
 
-  if ( opt.always_trust )
+  if ( opt.trust_model==TM_ALWAYS )
     {
       if( !opt.quiet )
         log_info(_("WARNING: Using untrusted key!\n"));
@@ -612,7 +519,7 @@ check_signatures_trust( PKT_signature *sig )
 		   " designated revoker!\n"));
       else
 	log_info(_("WARNING: This key has been revoked by its owner!\n"));
-      log_info(_("         This could mean that the signature is forgery.\n"));
+      log_info(_("         This could mean that the signature is forged.\n"));
       show_revocation_reason( pk, 0 );
     }
   else if ((trustlevel & TRUST_FLAG_SUB_REVOKED) ) 
@@ -807,7 +714,18 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned use )
      * list of the encrypt-to ones (we always trust them) */
     for( rov = remusr; rov; rov = rov->next ) {
 	if( !(rov->flags & 1) )
+	  {
 	    any_recipients = 1;
+
+	    if((rov->flags&2) && (PGP2 || PGP6 || PGP7 || PGP8))
+	      {
+		log_info(_("you may not use %s while in %s mode\n"),
+			 "--hidden-recipient",
+			 compliance_option_string());
+
+		compliance_failure();
+	      }
+	  }
 	else if( (use & PUBKEY_USAGE_ENC) && !opt.no_encrypt_to ) {
 	    pk = m_alloc_clear( sizeof *pk );
 	    pk->req_usage = use;
@@ -832,8 +750,17 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned use )
 		    r = m_alloc( sizeof *r );
 		    r->pk = pk; pk = NULL;
 		    r->next = pk_list;
-		    r->mark = 0;
+		    r->flags = (rov->flags&2)?1:0;
 		    pk_list = r;
+
+		    if(r->flags&1 && (PGP2 || PGP6 || PGP7 || PGP8))
+		      {
+			log_info(_("you may not use %s while in %s mode\n"),
+				 "--hidden-encrypt-to",
+				 compliance_option_string());
+
+			compliance_failure();
+		      }
 		}
 	    }
 	    else {
@@ -851,6 +778,8 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned use )
 	char *answer=NULL;
 	STRLIST backlog=NULL;
 
+	if(pk_list)
+	  any_recipients = 1;
 	def_rec = default_recipient();
 	have_def_rec = !!def_rec;
 	if( !have_def_rec )
@@ -866,12 +795,41 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned use )
 	    else if(backlog) {
 	      answer=pop_strlist(&backlog);
 	    }
-	    else {
+	    else
+	      {
+		PK_LIST iter;
+
+		tty_printf("\n");
+		tty_printf(_("Current recipients:\n"));
+		for(iter=pk_list;iter;iter=iter->next)
+		  {
+		    u32 keyid[2];
+
+		    keyid_from_pk(iter->pk,keyid);
+		    tty_printf("%4u%c/%s %s \"",
+			       nbits_from_pk(iter->pk),
+			       pubkey_letter(iter->pk->pubkey_algo),
+			       keystr(keyid),
+			       datestr_from_pk(iter->pk));
+
+		    if(iter->pk->user_id)
+		      tty_print_utf8_string(iter->pk->user_id->name,
+					    iter->pk->user_id->len);
+		    else
+		      {
+			size_t n;
+			char *p = get_user_id( keyid, &n );
+			tty_print_utf8_string( p, n );
+			m_free(p);
+		      }
+		    tty_printf("\"\n");
+		  }
+
 		answer = cpr_get_utf8("pklist.user_id.enter",
 			 _("\nEnter the user ID.  End with an empty line: "));
 		trim_spaces(answer);
 		cpr_kill_prompt();
-	    }
+	      }
 	    if( !answer || !*answer ) {
 	        m_free(answer);
 		break;
@@ -896,7 +854,7 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned use )
 			PK_LIST r = m_alloc( sizeof *r );
 			r->pk = pk; pk = NULL;
 			r->next = pk_list;
-			r->mark = 0;
+			r->flags = 0; /* no throwing default ids */
 			pk_list = r;
 		    }
 		    any_recipients = 1;
@@ -918,30 +876,10 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned use )
 			}
 			else {
 			    PK_LIST r;
-			    u32 keyid[2];
-
-			    keyid_from_pk( pk, keyid);
-			    tty_printf("Added %4u%c/%08lX %s \"",
-				       nbits_from_pk( pk ),
-				       pubkey_letter( pk->pubkey_algo ),
-				       (ulong)keyid[1],
-				       datestr_from_pk( pk ) );
-			    if(pk->user_id)
-			      tty_print_utf8_string(pk->user_id->name,
-						    pk->user_id->len);
-			    else
-			      {
-				size_t n;
-				char *p = get_user_id( keyid, &n );
-				tty_print_utf8_string( p, n );
-				m_free(p);
-			      }
-			    tty_printf("\"\n");
-
 			    r = m_alloc( sizeof *r );
 			    r->pk = pk; pk = NULL;
 			    r->next = pk_list;
-			    r->mark = 0;
+			    r->flags = 0; /* no throwing interactive ids */
 			    pk_list = r;
 			}
 			any_recipients = 1;
@@ -963,7 +901,7 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned use )
 	/* The default recipient may be disabled */
 	rc = get_pubkey_byname( pk, def_rec, NULL, NULL, 1 );
 	if( rc )
-	    log_error(_("unknown default recipient `%s'\n"), def_rec );
+	    log_error(_("unknown default recipient \"%s\"\n"), def_rec );
 	else if( !(rc=check_pubkey_algo2(pk->pubkey_algo, use)) ) {
 	  /* Mark any_recipients here since the default recipient
              would have been used if it wasn't already there.  It
@@ -976,7 +914,7 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned use )
 	    PK_LIST r = m_alloc( sizeof *r );
 	    r->pk = pk; pk = NULL;
 	    r->next = pk_list;
-	    r->mark = 0;
+	    r->flags = 0; /* no throwing default ids */
 	    pk_list = r;
 	  }
 	}
@@ -1036,7 +974,7 @@ build_pk_list( STRLIST rcpts, PK_LIST *ret_pk_list, unsigned use )
 			r = m_alloc( sizeof *r );
 			r->pk = pk; pk = NULL;
 			r->next = pk_list;
-			r->mark = 0;
+			r->flags = (remusr->flags&2)?1:0;
 			pk_list = r;
 		    }
 		}
@@ -1137,10 +1075,6 @@ algo_available( preftype_t preftype, int algo, void *hint )
 		  && algo != DIGEST_ALGO_SHA256))
 	return 0;
 
-      /* TIGER is not allowed any longer according to 2440bis. */
-      if( RFC2440 && algo == DIGEST_ALGO_TIGER )
-	return 0;
-
       return algo && !check_digest_algo( algo );
     }
   else if( preftype == PREFTYPE_ZIP )
@@ -1296,7 +1230,7 @@ select_algo_from_prefs(PK_LIST pk_list, int preftype, int request, void *hint)
 	 * algorithm 1 is also available (the ordering is not relevant
 	 * in this case). */
 	if( bits[0] & (1<<1) )
-	    i = 1;  /* yep; we can use compression algo 1 */
+	    i = 1; /* yep; we can use compression algo 1 */
     }
 
     /* "If you are building an authentication system, the recipient
@@ -1304,7 +1238,7 @@ select_algo_from_prefs(PK_LIST pk_list, int preftype, int request, void *hint)
        would be foolish to use a weak algorithm simply because the
        recipient requests it." RFC2440:13.  If we settle on MD5, and
        SHA1 is also available, use SHA1 instead.  Of course, if the
-       user intentinally chose MD5 (by putting it in their personal
+       user intentionally chose MD5 (by putting it in their personal
        prefs), then we should do what they say. */
 
     if(preftype==PREFTYPE_HASH &&
