@@ -15,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -29,15 +30,15 @@ static int
 build_index( MPI *exparray, int k, int i, int t )
 {
     int j, bitno;
-    int index = 0;
+    int idx = 0;
 
     bitno = t-i;
     for(j=k-1; j >= 0; j-- ) {
-	index <<= 1;
+	idx <<= 1;
 	if( mpi_test_bit( exparray[j], bitno ) )
-	    index |= 1;
+	    idx |= 1;
     }
-    return index;
+    return idx;
 }
 
 /****************
@@ -97,5 +98,3 @@ mpi_mulpowm( MPI res, MPI *basearray, MPI *exparray, MPI m)
 	mpi_free(G[i]);
     m_free(G);
 }
-
-
