@@ -1,5 +1,6 @@
 /* mpihelp-mul.c  -  MPI helper functions
- * Copyright (C) 1994, 1996, 1998, 1999, 2000 Free Software Foundation, Inc.
+ * Copyright (C) 1994, 1996, 1998, 1999,
+ *               2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -15,7 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  *
  * Note: This code is heavily based on the GNU MP Library.
  *	 Actually it's the same code with only minor changes in the
@@ -32,7 +34,7 @@
 #include <string.h>
 #include "mpi-internal.h"
 #include "longlong.h"
-#include "g10lib.h" /* g10_is_secure() */
+
 
 
 #define MPN_MUL_N_RECURSE(prodp, up, vp, size, tspace) \
@@ -353,7 +355,7 @@ mpihelp_mul_n( mpi_ptr_t prodp, mpi_ptr_t up, mpi_ptr_t vp, mpi_size_t size)
 	    mpih_sqr_n_basecase( prodp, up, size );
 	else {
 	    mpi_ptr_t tspace;
-	    secure = g10_is_secure( up );
+	    secure = m_is_secure( up );
 	    tspace = mpi_alloc_limb_space( 2 * size, secure );
 	    mpih_sqr_n( prodp, up, size, tspace );
 	    mpi_free_limb_space( tspace );
@@ -364,7 +366,7 @@ mpihelp_mul_n( mpi_ptr_t prodp, mpi_ptr_t up, mpi_ptr_t vp, mpi_size_t size)
 	    mul_n_basecase( prodp, up, vp, size );
 	else {
 	    mpi_ptr_t tspace;
-	    secure = g10_is_secure( up ) || g10_is_secure( vp );
+	    secure = m_is_secure( up ) || m_is_secure( vp );
 	    tspace = mpi_alloc_limb_space( 2 * size, secure );
 	    mul_n (prodp, up, vp, size, tspace);
 	    mpi_free_limb_space( tspace );
@@ -386,7 +388,7 @@ mpihelp_mul_karatsuba_case( mpi_ptr_t prodp,
 	if( ctx->tspace )
 	    mpi_free_limb_space( ctx->tspace );
 	ctx->tspace = mpi_alloc_limb_space( 2 * vsize,
-				       g10_is_secure( up ) || g10_is_secure( vp ) );
+				       m_is_secure( up ) || m_is_secure( vp ) );
 	ctx->tspace_size = vsize;
     }
 
@@ -399,8 +401,8 @@ mpihelp_mul_karatsuba_case( mpi_ptr_t prodp,
 	if( !ctx->tp || ctx->tp_size < vsize ) {
 	    if( ctx->tp )
 		mpi_free_limb_space( ctx->tp );
-	    ctx->tp = mpi_alloc_limb_space( 2 * vsize, g10_is_secure( up )
-						      || g10_is_secure( vp ) );
+	    ctx->tp = mpi_alloc_limb_space( 2 * vsize, m_is_secure( up )
+						      || m_is_secure( vp ) );
 	    ctx->tp_size = vsize;
 	}
 
@@ -420,7 +422,7 @@ mpihelp_mul_karatsuba_case( mpi_ptr_t prodp,
 	}
 	else {
 	    if( !ctx->next ) {
-		ctx->next = g10_xcalloc( 1, sizeof *ctx );
+		ctx->next = m_alloc_clear( sizeof *ctx );
 	    }
 	    mpihelp_mul_karatsuba_case( ctx->tspace,
 					vp, vsize,
@@ -449,7 +451,7 @@ mpihelp_release_karatsuba_ctx( struct karatsuba_ctx *ctx )
 	    mpi_free_limb_space( ctx->tp );
 	if( ctx->tspace )
 	    mpi_free_limb_space( ctx->tspace );
-	g10_free( ctx );
+	m_free( ctx );
     }
 }
 
